/*
** Graphics screen interface to SLICK!
**
** Version 1.0
**
** (c) by MacSoft 1991
**
** This function set allows you to use SLICK! to create graphics
** menu systems and windows. The technique is quite simple -
** the virtual screen buffer is created for screen writes and
** it is updated to physical screen when the keyboard scan
** occurs.
**
** Limitations:
**
** . AT or EGA/VGA required to use default update function
** . only fixed size fonts and fixed character positions could be used
**   because the image is actually managed in text mode
** . care should be taken by user that menus are not overwritten by
**   graphics routines
**
*/

#include "slick!.h"
#include "graphscr.h"

unsigned int far *gs_virtscrn;	/* virtual screen buffer used by SLICK! */
unsigned int far *gs_compscrn;	/* compare buffer for checking for changes */
void far cdecl (*gs_update)(int,int,unsigned int far *,int)=
			gs_def_update;		/* screen update routine */
static char far *fontpointer;	/* pointer to font information */
static int charsize;			/* bytes per character in this font */
static int far *bytespl=
	(int far *)0x0040004al;		/* bytes per screen line */
static char far *screenpointer=
	(char far *)0xa0000000l;		/* pointer to ega/vga screen buffer */

/*
** Initialize the system.
**
** Expect the size of the virtual text screen and graphics mode
** you're planning to use. It is essential to set the mode
** in _videocblk every time the physical video mode changes.
** Returns 1 of successful init, 0 on failure.
*/
int far cdecl gs_init(int cols,int rows,int mode)
{
	if (gs_virtscrn!=NULL)							/* if already active */
		gs_deinit();								/* deinitialize first */
	_videoparm();									/* detect video */
	_videocblk.base=0;								/* no base... */
	_videocblk.cols=cols;							/* set columns */
	_videocblk.rows=rows;							/* set rows */
	_videocblk.mode=mode;							/* set mode */
	_videocblk.snow&=0xfe;							/* and force no snow */
	rows*=cols;
	gs_virtscrn=_mymalloc(rows<<2);					/* get virtual screen */
	if (gs_virtscrn==NULL)
		return 0;									/* failed... */
	gs_compscrn=_mymalloc(rows<<2);					/* get second copy */
	if (gs_compscrn==NULL) {
		_myfree(gs_virtscrn);						/* failed, free first */
		gs_virtscrn=NULL;
		return 0;
	}
	_videocblk.base=(unsigned int)((unsigned long)gs_virtscrn>>16);
	for (cols=0;cols<rows;cols++) {
		gs_virtscrn[cols]=0x0000;					/* clear virtual screen */
		gs_compscrn[cols]=0x0000;					/* make compare same */
	}
	return 1;										/* success */
}

/*
** Deinitialize system.
*/
void far cdecl gs_deinit(void)
{
	_videocblk.base=0;								/* force redetect */
	if (gs_virtscrn!=NULL) {
		_myfree(gs_virtscrn);						/* free virtual screen */
		gs_virtscrn=NULL;
	}
	if (gs_compscrn!=NULL) {
		_myfree(gs_compscrn);						/* free compare screen */
		gs_compscrn=NULL;
	}
}

/*
** Set the mode in video control block to reflect actual video mode.
** This function should be called each time the physical video mode
** changes.
*/
void far cdecl gs_setmode(void)
{
unsigned char far *s=(unsigned char far *)0x00000449;
	_videocblk.mode=*s;
}

/*
** Refresh the screen.
** This function reflects the changes on the virtual screen onto physical
** screen.
** Returns 0 if refresh cannot be done or 1 if success.
*/
int far cdecl gs_refresh(void)
{
int x,y,z,xs,n;
	if (gs_virtscrn==NULL || gs_update==NULL)		/* if not initialized */
		return 0;
	for (y=0;y<_videocblk.rows;y++) {				/* scan all lines */
		n=0;										/* clear count */
		z=y*_videocblk.cols;						/* calculate row offset */
		for (x=0;x<_videocblk.cols;x++) {			/* scan one row */
			if (gs_compscrn[z+x]!=gs_virtscrn[z+x]) {
				gs_compscrn[z+x]=gs_virtscrn[z+x];	/* set them equal */
				if (!n)								/* if first in the burst */
					xs=x;							/* remember the begin */
				n++;								/* bump count */
			}
			else {									/* if equal */
				if (n) {							/* but we had diff */
					(*gs_update)(xs,y,&gs_compscrn[z+xs],n); /* update */
					n=0;							/* clear count */
				}
			}
		}
		if (n)										/* if  end of line dif */
			(*gs_update)(xs,y,&gs_compscrn[z+xs],n);	/* update screen */
	}
	return 1;
}

/*
** Set font information.
*/
void gs_setfont(char far *font,int size)
{
	fontpointer=font;
	charsize=size;
}

/*
** Default update routine. Works only for 4-plane ega/vga modes.
*/
void far cdecl gs_def_update(int x,int y,unsigned int far *buf,int n)
{
	while (n) {
		gs_writechar(x++,y,*buf++);
		n--;
	}
}

void gs_writechar(int x,int y,int charatt)
{
int i;
unsigned int u,xc;
char far *fc,fgcolor,bgcolor;
	u=((unsigned)y*(unsigned)charsize)*(*bytespl)+(unsigned)x;
	fc=fontpointer+((unsigned)(charatt&0xff)*(unsigned)charsize);
	fgcolor=(charatt>>8)&0x0f;
	bgcolor=(charatt>>12);
	outpw(0x3ce,0x0a05);
	outpw(0x3ce,0x0003); /* 0x1803 */
	outpw(0x3ce,0x0007);
	for (i=0;i<charsize;i++) {
		/* plot one byte from *fc to screenpointer[u] */
		xc=(((unsigned)*fc)<<8)|0x0008;
		outpw(0x3ce,xc);
		screenpointer[u]&=fgcolor;
		outpw(0x3ce,(xc^0xff00));
		screenpointer[u]&=bgcolor;
		u+=(unsigned)(*bytespl);
		fc++;
	}
	outpw(0x3ce,0xff08);
	outpw(0x3ce,0x0005);
	outpw(0x3ce,0x0003);
	outpw(0x3ce,0x0f07);
}
