#include "slick!.h"
#include "menueng.h"
#include "graphscr.h"
#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <mem.h>
#include <time.h>

int far scankey(ME_MENU far *m);

void interrupt (*oldbreak)(void);

struct democolors {
	ME_COLORS menucol;
	word desktop;
	word window;
	byte help;
	byte errmsg;
};

struct democolors colors = {
	{
		0x07,	/* normal */
		0x0a,	/* highlited char */
		0x4f,	/* cursor bar */
		0x4a,	/* hilited character in cursor bar */
		0x04,	/* disabled item */
		0x0c,	/* cursor bar on disabled item */
	},
	0x1f1f,		/* desktop */
	0x2f2f,		/* pulldown windows */
	0x0f,		/* help messages */
	0x8c		/* error messages */
};

struct democolors monoset = {
	{
		0x07,
		0x0f,
		0x70,
		0x7f,
		0x07,
		0x01,
	},
	0x0f0f,
	0x0f0f,
	0x0f,
	0x8c
};

/*
** Write help message.
*/
void far helpout(ME_MENU far *m)
{
	_vputscp(0,_videocblk.rows-1,m->items[m->def-1].help,colors.help,
				_videocblk.cols);
}


/*
** Menu system error handler.
*/
void far menuerr(ME_MENU far *m,int error)
{
	m->def+=0;
	_DH=_videocblk.rows-1;
	_DL=0;
	_BH=0;
	_AH=2;
	__int__(0x10);
	printf("Menu System Error - ");
	switch (error) {
		case ME_ERRSTARTUP:
			printf("Startup");
			break;
		case ME_ERRCLEANUP:
			printf("Cleanup");
			break;
		default:
			printf("Unknown (%d)",error);
			break;
	}
	printf(". Press any key to abort...");
	_recolor(0,_videocblk.rows-1,80,colors.errmsg);
	waitkey();
	exit(error);
}

/*
** Main menu startup - draw desktop.
*/
int far mainstart(ME_MENU far *m,int x1,int y1,int x2,int y2)
{
	x1=x1+y1+x2+y2+m->def;
	x1=makewnd(0,0,_videocblk.cols-1,_videocblk.rows-1,"      ",
		colors.desktop,80);
	if (x1)
		cursoroff();	/* hide cursor if succeeded */
	return x1;
}

/*
** Startup/cleanup for menus.
*/
int far menustart(ME_MENU far *m,int x1,int y1,int x2,int y2)
{
	m->def+=0;
	return makewnd(x1-1,y1-1,x2+1,y2+1,DOUBLEFRAME,colors.window,80);
}

int far menuclean(ME_MENU far *m)
{
	m->def+=0;
	return closewnd();
}

/*
** This function will be called from pulldown.
*/
void far demofunc(ME_MENU far *m)
{
char s[80];
	sprintf(s,"You chose the \"%s\" from the menu",m->items[m->def-1].text);
	statwnd(s,FATFRAME,colors.window);
	scankey(m);
	closewnd();
}

ME_ITEMS mixed_img [] = {
	{ 10, 7,7,ME_IT_FUNC,2, 0,0,"Tic",demofunc,""},
	{ 20, 7,7,ME_IT_FUNC,2, 0,0,"Tac",demofunc,""},
	{ 30, 7,7,ME_IT_FUNC,2, 0,0,"Toe",demofunc,""},
	{ 15, 10,13,ME_IT_FUNC,2, 0,0,"Crossword",demofunc,""},
	{ 15, 11,9,ME_IT_FUNC,2, 1,0,"Chess",demofunc,""},
	{ 17, 13,11,ME_IT_NORM,2, 0,0,"Nothing",NULL,""},

};

ME_MENU mixed= { 6,1,ME_S_MOVE_UP|ME_S_MOVE_DN,
			&colors.menucol,mixed_img,helpout,
				menustart,menuclean,menuerr,scankey,NULL };

ME_ITEMS pull_img [] = {
	{ 1, 2,20,ME_IT_FUNC,2, 0,0,"First",demofunc,"This will call the function"},
	{ 1, 3,20,ME_IT_FUNC,2, 0,0,"Second",demofunc,"This will call the function also"},
	{ 1, 4,20,ME_IT_FUNC,2, 0,0,"Third",demofunc,"And this calls a function"},
	{ 1, 5,20,ME_IT_DOOR,2, 0,0,"Pulldown",&mixed,"This is a mixed layout menu"},
	{ 1, 6,20,ME_IT_NORM,2, 0,0,"Out",NULL,"This will return to main menu"},

};

ME_MENU pull= { 5,1,ME_S_MOVE_UP|ME_S_MOVE_DN|ME_S_LTRT_QT,
			&colors.menucol,pull_img,helpout,
				menustart,menuclean,menuerr,scankey,NULL };

/*
** Main menu image & structure.
*/
ME_ITEMS main_img [] = {
	{ 0, 0,14,ME_IT_DOOR,1, 0,0,			"Show me more",&pull,"More choices in here"},
	{14, 0,15,ME_IT_FUNC|ME_IT_DIS,1, 0,0,	"Disabled item",NULL,"Second item"},
	{29, 0, 6,ME_IT_NORM,1, 0,0,			"Quit",NULL,"This is the way out"},
};


ME_MENU mainmenu={ 3,1,ME_S_MOVE_LT|ME_S_MOVE_RT|ME_S_DNEQUCR,
			&colors.menucol,main_img,helpout,
				mainstart,NULL,menuerr,scankey,NULL };


void interrupt newbreak(void)
{
}

/*
** Wait for user response.
**
** This is demonstration what we can do in here -
** this function filters out ESCapes pressed while in main menu.
*/
int far scankey(ME_MENU far *m)
{
int key;
	if (testkey())		/* ignore any pending keystrokes */
		waitkey();		/* just in case... */
	_mousepress(LEFT,&key,&key,&key);	/* ignore pending presses */
	gs_refresh();
	do {
		key=gcuakey(m);
	} while (key==ME_ESC_KEY && m==&mainmenu);	/* filter ESCapes */
	return key;			/* return the key */
}

/*
** This is general clenup function -
**
** We will release all memory and interrupt vectors,
** this is also a right place to deinitialize mouse driver.
*/
void cleanup(void)
{
	while (closewnd());			/* close all windows */
	gs_deinit();
	_mousedeinit();				/* deinitialize mouse */
	if (oldbreak!=NULL)			/* if we have the break vector already */
		setvect(0x1b,oldbreak);	/* then put it back */
}

char far *getfptr(void)
{
	asm {
		push	bp
		mov		ax,1130h
		mov		bh,2
		int		10h
		mov		ax,bp
		pop		bp
	}
	return MK_FP(_ES,_AX);
}


/*
** Main's body is real simple now - just set up everything and
** let the menu engine do the job.
*/
void main(void)
{
	atexit(cleanup);		/* first set up clenup function */
	oldbreak=getvect(0x1b);	/* get break vector */
	setvect(0x1b,newbreak);	/* and replace it with ours */
	gs_init(80,23,0x10);

	gs_setfont(getfptr(),14);
	_AX=0x10;
	__int__(0x10);
	_mouseinit(STDMOUSE);	/* this will also set up video control block */
	if (_videocblk.mode==7)	/* so we can check video mode here */
		movmem(&monoset,&colors,sizeof(struct democolors));
	menueng(ME_MAKE,&mainmenu);
	exit(0);
}


/*
#define WI 80
#define HE 24
#define MO 0x10
#define CO 16

char * frames[] = {
	SINGLEFRAME,
	DOUBLEFRAME,
	FATFRAME,
	SINGLEFRAME,
	DOUBLEFRAME,
	FULLFRAME
};
void print(char *s)
{
	while (*s)
		putcwnd(*s++);
}


void main(void)
{
int om;
int i,x1,y1,x2,y2,n;
time_t t;
	if (gs_init(WI,HE,MO)) {
		_myseed(time(&t));
		_AH=0x0f;
		__int__(0x10);
		om=_AX&0xff;
		_AX=MO;
		__int__(0x10);	/* switch to graphics */
		gs_setfont(getfptr(),14);
		makewnd(5,10,WI-5,14,DOUBLEFRAME,0x4243,80);
		prtcwnd(1,"I just love the way I did it...",_myrand()%CO);
		gs_refresh();
		waitkey();
		for (i=0;i<100;i++) {
			x1=_myrand()%(WI-2);
			x2=_myrand()%(WI-2);
			if (x1>x2) {
				n=x1,x1=x2,x2=n;
			}
			if (x2==x1)
				x2++;
			y1=_myrand()%(HE-2);
			y2=_myrand()%(HE-2);
			if (y1>y2) {
				n=y1,y1=y2,y2=n;
			}
			if (y2==y1)
				y2++;
			makewnd(x1,y1,x2,y2,frames[_myrand()%6],
				0x0240|(_myrand()%(CO-1)+1),80);
			if (y2>y1+1 && x2>x1+1)
				print(	"Multicolored graphics windows\r\n"
						"Using SLICK! companion package\r\n"
						"GRAPHSCR - the interface to graphics");
			gs_refresh();
		}
		waitkey();
		while (closewnd())
			gs_refresh();
		waitkey();
		gs_deinit();
		_AX=om;
		__int__(0x10);
	}
}

*/
